#ifdef __cplusplus
extern "C" {
#endif

#include "SimUtils.h"
#include "Fusion_SLAM_types.h"
//#include "fusion_ext.h"

static int getNextElement(char* line, char** element_ptr);
static void readNextLine(FILE** fptr_ptr, CSVRow* row);
static void moveFilePtrToNextLine(FILE** fptr_ptr);

static void fillVehDynFromCSV(CSVRow row, VehDyn_t* veh_data_row_ptr);
// static void fillExtObjectListFromCSV(CSVRow row, ExtObjectList_t* radar_data_ptr);
static void fillLidarObjectListFromCSV(CSVRow row, Lidar_input_lists* lidar_data_ptr);
static void fillCamObjListFromCSV(CSVRow row, Camera_input_lists* camera_data_ptr);

/**
 * @brief       Writes fusion output coloumn names
 * 
 * A CSV file pointer is accepted and fusion output coloumn names
 * are written to the file
 * 
 * @param[in]   fptr pointer to file pointer
 * 
 * @return      Nothing is returned
 * 
*/
void writeFusionOutputCols(FILE** fptr){
    fprintf(*fptr, "TueFusionObjectOutput_uiVersionNumber,TueFusionObjectOutput_sSigHeader_uiTimeStamp,TueFusionObjectOutput_sSigHeader_uiMeasurementCounter,TueFusionObjectOutput_sSigHeader_uiCycleCounter,TueFusionObjectOutput_sSigHeader_eSigStatus,TueFusionObjectOutput_HeaderObjList_iNumOfUsedObjects");
    for(int j = 0; j < BSW_NR_PRIVOBJECTS; j++)
        fprintf(*fptr, ",TueFusionObjectOutput_HeaderObjList_iDistXSortedObjectList_%d", j);
    fprintf(*fptr, ",TueFusionObjectOutput_sSceneDescription_uiTunnelProb,TueFusionObjectOutput_sSceneDescription_uiFilteredTimeIndicator");
    for(int j = 0; j < BSW_NR_PRIVOBJECTS; j++){
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_ObjectId", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fDistX", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fDistXStd", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fDistY", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fDistYStd", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fVrelX", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fVrelXStd", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fVrelY", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fVrelYStd", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fArelX", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fArelXStd", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fArelY", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Kinematic_fArelYStd", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Geometry_fWidth", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Geometry_fLength", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Geometry_fOrientation", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Geometry_fOrientationStd", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Geometry_fOrientationValid", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Attributes_eDynamicProperty", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Attributes_uiStoppedConfidence", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Attributes_eAbsMovingState", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Attributes_eClassification", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Attributes_uiClassConfidence", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_General_fLifeTime", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_General_fTimeStamp", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_General_eObjMaintenanceState", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_General_cObjMergingID", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Qualifiers_fProbabilityOfExistence", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Qualifiers_ucObstacleProbability", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Qualifiers_uMeasuredTargetFrequencyNear", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Qualifiers_uMeasuredTargetFrequencyFar", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_SensorSpecific_fRCS", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_SensorSpecific_bCamConfirmed", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_ACCPresel_ucAccObjQuality", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_ACCPresel_ucAccSelBits", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_EBAPresel_eEbaInhibitionMask", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_EBAPresel_ucEbaMovingObjQuality", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_EBAPresel_eEbaHypCat", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_EBAPresel_bCrossingPedEbaPresel", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Legacy_fMaxAccelY", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Legacy_fRCSTargetThresholdUncomp", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Legacy_uiLifeTime", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_Legacy_eDynamicSubProperty", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_au16SensorID_0", j);
        fprintf(*fptr, ",TueFusionObjectOutput_Objects_%d_au16SensorID_1", j);
    }
    fprintf(*fptr, "\n");
}

/**
 * @brief       Writes output of one fusion cycle to CSV file
 * 
 * A CSV file pointer and ObjectList_t read-only pointer is
 * accepted and its data is written to the CSV file. Coloumn
 * orderings are preserved and is same as coloumn names in writeFusionOutputCols function
 * 
 * @param[in]   fptr pointer to file pointer
 * @param[in]   data read-only pointer of fusion output result
 * 
 * @return      Nothing is returned
 * 
*/
void writeFusionOutputToFile(FILE** fptr, const ObjectList_t* const data){
    fprintf(*fptr, "%u", data->uiVersionNumber);
    fprintf(*fptr, ",%u", data->sSigHeader.uiTimeStamp);
    fprintf(*fptr, ",%hu", data->sSigHeader.uiMeasurementCounter);
    fprintf(*fptr, ",%hu", data->sSigHeader.uiCycleCounter);
    fprintf(*fptr, ",%u", data->sSigHeader.eSigStatus);

    // All char values are interpreted as integer values
    fprintf(*fptr, ",%d", data->HeaderObjList.iNumOfUsedObjects);
    for(int j = 0; j < BSW_NR_PRIVOBJECTS; j++)
        fprintf(*fptr, ",%d", data->HeaderObjList.iDistXSortedObjectList[j]);

    fprintf(*fptr, ",%u", data->sSceneDescription.uiTunnelProb);
    fprintf(*fptr, ",%u", data->sSceneDescription.uiFilteredTimeIndicator);

    for(int j = 0; j < BSW_NR_PRIVOBJECTS; j++){
        fprintf(*fptr, ",%u", data->Objects[j].ObjectId);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fDistX);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fDistXStd);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fDistY);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fDistYStd);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fVrelX);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fVrelXStd);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fVrelY);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fVrelYStd);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fArelX);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fArelXStd);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fArelY);
        fprintf(*fptr, ",%f", data->Objects[j].Kinematic.fArelYStd);
        fprintf(*fptr, ",%f", data->Objects[j].Geometry.fWidth);
        fprintf(*fptr, ",%f", data->Objects[j].Geometry.fLength);
        fprintf(*fptr, ",%f", data->Objects[j].Geometry.fOrientation);
        fprintf(*fptr, ",%f", data->Objects[j].Geometry.fOrientationStd);
        fprintf(*fptr, ",%f", data->Objects[j].Geometry.fOrientationValid);
        fprintf(*fptr, ",%u", data->Objects[j].Attributes.eDynamicProperty);
        fprintf(*fptr, ",%u", data->Objects[j].Attributes.uiStoppedConfidence);
        fprintf(*fptr, ",%u", data->Objects[j].Attributes.eAbsMovingState);
        fprintf(*fptr, ",%u", data->Objects[j].Attributes.eClassification);
        fprintf(*fptr, ",%u", data->Objects[j].Attributes.uiClassConfidence);
        fprintf(*fptr, ",%f", data->Objects[j].General.fLifeTime);
        fprintf(*fptr, ",%u", data->Objects[j].General.fTimeStamp);
        fprintf(*fptr, ",%u", data->Objects[j].General.eObjMaintenanceState);
        fprintf(*fptr, ",%u", data->Objects[j].General.cObjMergingID);
        fprintf(*fptr, ",%f", data->Objects[j].Qualifiers.fProbabilityOfExistence);
        fprintf(*fptr, ",%u", data->Objects[j].Qualifiers.ucObstacleProbability);
        fprintf(*fptr, ",%u", data->Objects[j].Qualifiers.uMeasuredTargetFrequencyNear);
        fprintf(*fptr, ",%u", data->Objects[j].Qualifiers.uMeasuredTargetFrequencyFar);
        fprintf(*fptr, ",%f", data->Objects[j].SensorSpecific.fRCS);
        fprintf(*fptr, ",%u", data->Objects[j].SensorSpecific.bCamConfirmed);
        fprintf(*fptr, ",%u", data->Objects[j].ACCPresel.ucAccObjQuality);
        fprintf(*fptr, ",%u", data->Objects[j].ACCPresel.ucAccSelBits);
        fprintf(*fptr, ",%hu", data->Objects[j].EBAPresel.eEbaInhibitionMask);
        fprintf(*fptr, ",%u", data->Objects[j].EBAPresel.ucEbaMovingObjQuality);
        fprintf(*fptr, ",%u", data->Objects[j].EBAPresel.eEbaHypCat);
        fprintf(*fptr, ",%u", data->Objects[j].EBAPresel.bCrossingPedEbaPresel);
        fprintf(*fptr, ",%f", data->Objects[j].Legacy.fMaxAccelY);
        fprintf(*fptr, ",%f", data->Objects[j].Legacy.fRCSTargetThresholdUncomp);
        fprintf(*fptr, ",%hu", data->Objects[j].Legacy.uiLifeTime);
        fprintf(*fptr, ",%u", data->Objects[j].Legacy.eDynamicSubProperty);
        fprintf(*fptr, ",%hu", data->Objects[j].au16SensorID[0]);
        fprintf(*fptr, ",%hu", data->Objects[j].au16SensorID[1]);
    }
    fprintf(*fptr, "\n");
}

/**
 * @brief       reads the ego vehicle data csv file
 * 
 * Vehicle fake data is read and a list of VehDyn_t elements is filled. Each
 * element corresponds to ego vehicle dynamic at time instant t
 * 
 * @param[in]   file_name ego vehicle fake data file name
 * @param[in]   veh_data pointer to an array of ego vehicle dynamic data
 * @param[in]   cols number of coloumns to be read
 * 
 * @return      Nothing is returned
 * 
 * @see VehDyn_t
*/
void readEgoVehicleData(char* file_name, VehDyn_t* veh_data, size_t cols){
    FILE* fptr = fopen(file_name, "r");
    if(fptr == NULL){
        LogWrite("Unable to open %s file!\n", file_name);
        return;
    }
    char line[LINE_BUFF_SIZE];
    char elem[CSV_ELEMENT_SIZE];
    char* line_ptr = &line[0];
    char* elem_ptr = &elem[0];
    int elem_len = 0;
    int veh_data_idx = 0;

    CSVRow row;
    row.len = cols;
    row.elements = malloc(cols * sizeof(char*));
    #if (TUE_PRV_RUNTIME_POINTER_CHECK == STD_ON)
        if (NULL_PTR == row.elements)
        {
            u32Success = TUEOBJFUSN_ERROR_NULL_POINTER_EXCEPTION;
        }
        else
    #endif
    for(int i = 0; i < cols; i++)
        row.elements[i] = malloc(CSV_ELEMENT_SIZE * sizeof(char));
    // Do not read first line as it only contains coloumn names
    moveFilePtrToNextLine(&fptr);
    while(TRUE){
        // read next line of file
        readNextLine(&fptr, &row);
        if(feof(fptr))
            break;
        // fill VehDyn_t** data i'th element
        fillVehDynFromCSV(row, &veh_data[veh_data_idx++]);
    }

    for(int i = 0; i < cols; i++)
        free(row.elements[i]);
    free(row.elements);

    // Reset simulation timestamp
    ego_veh_ts.idx = 0;
    fclose(fptr);
}

// /**
//  * @brief       reads the Radar sensor data from csv file
//  * 
//  * Radar fake data is read and a list of ExtObjectList_t elements is filled. Each
//  * element corresponds to Radar measurement at instant t
//  * 
//  * @param[in]   file_name Radar fake data file name
//  * @param[in]   radar_data_ptr pointer to an array of Radar data
//  * @param[in]   cols number of coloumns to be read
//  * 
//  * @return      Nothing is returned
//  * 
//  * @see ExtObjectList_t
// */
// void readRadarSensorData(char* file_name, ExtObjectList_t* radar_data_ptr, size_t cols){
//     FILE* fptr = fopen(file_name, "r");
//     if(fptr == NULL){
//         LogWrite("Unable to open %s file!\n", file_name);
//         return;
//     }
//     char line[LINE_BUFF_SIZE];
//     char elem[CSV_ELEMENT_SIZE];
//     char* line_ptr = &line[0];
//     char* elem_ptr = &elem[0];
//     int elem_len = 0;
//     int radar_data_idx = 0;

//     CSVRow row;
//     row.len = cols;
//     row.elements = malloc(cols * sizeof(char*));
//     #if (TUE_PRV_RUNTIME_POINTER_CHECK == STD_ON)
//         if (NULL_PTR == row.elements)
//         {
//             u32Success = TUEOBJFUSN_ERROR_NULL_POINTER_EXCEPTION;

//         }
//         else
//     #endif
//     for(int i = 0; i < cols; i++)
//         row.elements[i] = malloc(CSV_ELEMENT_SIZE * sizeof(char));
//     // Do not read first line as it only contains coloumn names
//     moveFilePtrToNextLine(&fptr);
//     while(TRUE){
//         // read next line of file
//         readNextLine(&fptr, &row);
//         if(feof(fptr))
//             break;
//         // fill VehDyn_t** data i'th element
//         fillExtObjectListFromCSV(row, &radar_data_ptr[radar_data_idx++]);
//     }

//     radar_ts.idx = 0;
//     for(int i = 0; i < cols; i++)
//         free(row.elements[i]);
//     free(row.elements);

//     fclose(fptr);
// }

/**
 * @brief       reads the Lidar sensor data from csv file
 * 
 * Lidar fake data is read and a list of Lidar_input_lists elements is filled. Each
 * element corresponds to Lidar measurement at instant t
 * 
 * @param[in]   file_name Lidar fake data file name
 * @param[in]   lidar_data_ptr pointer to an array of Lidar data
 * @param[in]   cols number of coloumns to be read
 * 
 * @return      Nothing is returned
 * 
 * @see Lidar_input_lists
*/
/*void readLidarSensorData(char* file_name, Lidar_input_lists* lidar_data_ptr, size_t cols){
    FILE* fptr = fopen(file_name, "r");
    if(fptr == NULL){
        LogWrite("Unable to open %s file!\n", file_name);
        return;
    }
    char line[LINE_BUFF_SIZE];
    char elem[CSV_ELEMENT_SIZE];
    char* line_ptr = &line[0];
    char* elem_ptr = &elem[0];
    int elem_len = 0;
    int lidar_data_idx = 0;

    CSVRow row;
    row.len = cols;
    row.elements = malloc(cols * sizeof(char*));
    #if (TUE_PRV_RUNTIME_POINTER_CHECK == STD_ON)
        if (NULL_PTR == row.elements)
        {
            u32Success = TUEOBJFUSN_ERROR_NULL_POINTER_EXCEPTION;

        }
        else
    #endif
    for(int i = 0; i < cols; i++)
        row.elements[i] = malloc(CSV_ELEMENT_SIZE * sizeof(char));
    // Do not read first line as it only contains coloumn names
    moveFilePtrToNextLine(&fptr);
    while(TRUE){
        // read next line of file
        readNextLine(&fptr, &row);
        if(feof(fptr))
            break;
        // fill LidarObjectList_t** data i'th element
        fillLidarObjectListFromCSV(row, &lidar_data_ptr[lidar_data_idx++]);
    }

    lidar_ts.idx = 0;
    for(int i = 0; i < cols; i++)
        free(row.elements[i]);
    free(row.elements);

    fclose(fptr);
}
*/

/**
 * @brief       reads the Camera sensor data from csv file
 * 
 * Camera fake data is read and a list of Camera_input_lists elements is filled. Each
 * element corresponds to Camera measurement at instant t
 * 
 * @param[in]   file_name Camera fake data file name
 * @param[in]   cam_data_ptr pointer to an array of Camera data
 * @param[in]   cols number of coloumns to be read
 * 
 * @return      Nothing is returned
 * 
 * @see Camera_input_lists
*/
void readCameraSensorData(char* file_name, Camera_input_lists* cam_data_ptr, size_t cols){
    FILE* fptr = fopen(file_name, "r");
    if(fptr == NULL){
        LogWrite("Unable to open %s file!\n", file_name);
        return;
    }
    char line[LINE_BUFF_SIZE];
    char elem[CSV_ELEMENT_SIZE];
    char* line_ptr = &line[0];
    char* elem_ptr = &elem[0];
    int elem_len = 0;
    int camera_data_idx = 0;

    CSVRow row;
    row.len = cols;
    row.elements = malloc(cols * sizeof(char*));
    #if (TUE_PRV_RUNTIME_POINTER_CHECK == STD_ON)
        if (NULL_PTR == row.elements)
        {
            u32Success = TUEOBJFUSN_ERROR_NULL_POINTER_EXCEPTION;
        }
        else
    #endif
    
    for(int i = 0; i < cols; i++)
        row.elements[i] = malloc(CSV_ELEMENT_SIZE * sizeof(char));

    // Do not read first line as it only contains coloumn names
    int tmp_idx = 1;
    moveFilePtrToNextLine(&fptr);
    while(TRUE){
        // read next line of file
        readNextLine(&fptr, &row);
        if(feof(fptr))
            break;
        // fill VehDyn_t** data i'th element
        fillCamObjListFromCSV(row, &cam_data_ptr[camera_data_idx++]);
    }

    camera_ts.idx = 0;
    
    for(int i = 0; i < cols; i++)
        free(row.elements[i]);
    free(row.elements);

    fclose(fptr);
}

/**
 * @brief       populates a Camera_input_lists object from corresponding row in CSV file
 * 
 * After reading one row of CSV file, this function is used to fill each element of
 * Camera_input_lists from corresponding row of CSV file
 * 
 * @param[in]   row one row of CSV file
 * @param[in]   cam_data_ptr pointer to Camera_input_lists variable that is being populated
 * 
 * @return      Nothing is returned
 * 
*/
static void fillCamObjListFromCSV(CSVRow row, Camera_input_lists* cam_data_ptr){
    camera_ts.timestamp[camera_ts.idx++] = (uint32) (atoi(row.elements[0]));

    //cam_data_ptr->uiVersionNumber = (uint32) (atoi(row.elements[1]));

    cam_data_ptr->Cameraobjlists->last_measure_update_timestamp_ns= (uint32) (atoi(row.elements[2]));
    //cam_data_ptr->sSigHeader.uiMeasurementCounter = (uint16) (atoi(row.elements[3]));
    //cam_data_ptr->sSigHeader.uiCycleCounter = (uint16) (atoi(row.elements[4]));
    //cam_data_ptr->sSigHeader.eSigStatus = (uint8) (atoi(row.elements[5]));

    for(int i = 0; i < TUE_CAMERA_RAW_OBJECT_NUM; i++){
        cam_data_ptr->Cameraobjlists->Objects[i].track_id = (uint32) (atoi(row.elements[6+i*36]));
        
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fDist.data[0] = (float32) (atof(row.elements[7+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fCovDist.data[0][0] = (float32) (atof(row.elements[8+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fDist.data[1] = (float32) (atof(row.elements[9+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fCovDist.data[1][1] = (float32) (atof(row.elements[10+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fVel.data[0] = (float32) (atof(row.elements[11+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fCovVel.data[0][0] = (float32) (atof(row.elements[12+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fVel.data[1] = (float32) (atof(row.elements[13+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fCovVel.data[1][1]= (float32) (atof(row.elements[14+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fAcc.data[0] = (float32) (atof(row.elements[15+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fCovAcc.data[0][0] = (float32) (atof(row.elements[16+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fAcc.data[0] = (float32) (atof(row.elements[17+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fCovAcc.data[0][0] = (float32) (atof(row.elements[18+i*36]));
        //cam_data_ptr->Cameraobjlists->Objects[i].Kinematic.fOrientation = (float32) (atof(row.elements[19+i*36]));
        //cam_data_ptr->aObject[i].Kinematic.eRefpointLocation = (uint8) (atoi(row.elements[20+i*36]));
        
        cam_data_ptr->Cameraobjlists->Objects[i].Geometry.height = (float32) (atof(row.elements[21+i*36]));
        //cam_data_ptr->aObject[i].Geometry.fOffsetToGround = (float32) (atof(row.elements[22+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Geometry.length = (float32) (atof(row.elements[23+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Geometry.width = (float32) (atof(row.elements[24+i*36]));
        
        
        cam_data_ptr->Cameraobjlists->Objects[i].Classification.type_confidence = (float32) (atoi(row.elements[25+i*36]));
        //cam_data_ptr->aObject[i].Classification.ObjectType = (uint8) (atoi(row.elements[26+i*36]));

        //cam_data_ptr->aObject[i].Attributes.eStatusBrakeLight = (uint8) (atoi(row.elements[27+i*36]));
        //cam_data_ptr->aObject[i].Attributes.uiBrakeLightConfidence = (uint8) (atoi(row.elements[28+i*36]));
        //cam_data_ptr->aObject[i].Attributes.eStatusTurnIndicator = (uint8) (atoi(row.elements[29+i*36]));
        //cam_data_ptr->aObject[i].Attributes.uiTurnIndicatorConfidence = (uint8) (atoi(row.elements[30+i*36]));
        //cam_data_ptr->aObject[i].Attributes.eAssociatedLane = (uint8) (atoi(row.elements[31+i*36]));
        //cam_data_ptr->aObject[i].Attributes.uiAssociatedLaneConfidence = (uint8) (atoi(row.elements[32+i*36]));
        //cam_data_ptr->aObject[i].Attributes.eLaneChangeManeuver = (uint8) (atoi(row.elements[33+i*36]));
        //cam_data_ptr->aObject[i].Attributes.percentageOwnDrivingLane = (uint8) (atoi(row.elements[34+i*36]));
       //cam_data_ptr->aObject[i].Attributes.percentageSideDrivingLaneLeft = (uint8) (atoi(row.elements[35+i*36]));
       //cam_data_ptr->aObject[i].Attributes.percentageSideDrivingLaneRight = (uint8) (atoi(row.elements[36+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Attributes.maintenance_status = (uint16) (atoi(row.elements[37+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Attributes.existence_confidence = (float32) (atoi(row.elements[38+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Attributes.motion_status = (uint16) (atoi(row.elements[39+i*36]));
        //cam_data_ptr->Cameraobjlists->Objects[i].Attributes.eObjectOcclusion = (uint8) (atoi(row.elements[40+i*36]));
        cam_data_ptr->Cameraobjlists->Objects[i].Attributes.track_age_time_second = (float32) (atoi(row.elements[40+i*36]));
    }
    // IDX = 870
   // for(int i = 0; i < 4; i++){
    //    cam_data_ptr->sLaneLines[i].Type = (uint8) (atoi(row.elements[870+i*12]));
    //   cam_data_ptr->sLaneLines[i].Quality = (uint8) (atoi(row.elements[871+i*12]));
    //    cam_data_ptr->sLaneLines[i].Position = (float) (atof(row.elements[872+i*12]));
    //    cam_data_ptr->sLaneLines[i].Curvature = (float) (atof(row.elements[873+i*12]));
   //     cam_data_ptr->sLaneLines[i].CurvatureDer = (float) (atof(row.elements[874+i*12]));
   //     cam_data_ptr->sLaneLines[i].WidthMarking = (float) (atof(row.elements[875+i*12]));
   //     cam_data_ptr->sLaneLines[i].HeadingAngle = (float) (atof(row.elements[876+i*12]));
   //     cam_data_ptr->sLaneLines[i].ViewRangeStart = (float) (atof(row.elements[877+i*12]));
   //     cam_data_ptr->sLaneLines[i].ViewRangeEnd = (float) (atof(row.elements[878+i*12]));
   //     cam_data_ptr->sLaneLines[i].LineCrossing = (uint8) (atoi(row.elements[879+i*12]));
     //   cam_data_ptr->sLaneLines[i].LineMarkColor = (uint8) (atoi(row.elements[880+i*12]));
       // cam_data_ptr->sLaneLines[i].CanMsgEnd = (uint8) (atoi(row.elements[881+i*12]));
    
}

// /**
//  * @brief       populates a ExtObjectList_t object from corresponding row in CSV file
//  * 
//  * After reading one row of CSV file, this function is used to fill each element of
//  * ExtObjectList_t from corresponding row of CSV file
//  * 
//  * @param[in]   row one row of CSV file
//  * @param[in]   radar_data_ptr pointer to ExtObjectList_t variable that is being populated
//  * 
//  * @return      Nothing is returned
//  * 
// */
// static void fillExtObjectListFromCSV(CSVRow row, ExtObjectList_t* radar_data_ptr){
//     radar_ts.timestamp[radar_ts.idx++] = (uint32) (atoi(row.elements[0]));

//     radar_data_ptr->uiVersionNumber = (uint32) (atoi(row.elements[1]));

//     radar_data_ptr->sSigHeader.uiTimeStamp = (uint32) (atoi(row.elements[2]));
//     // Set simulation timestamp
//     radar_data_ptr->sSigHeader.uiMeasurementCounter = (uint16) (atoi(row.elements[3]));
//     radar_data_ptr->sSigHeader.uiCycleCounter = (uint16) (atoi(row.elements[4]));
//     radar_data_ptr->sSigHeader.eSigStatus = (uint8) (atoi(row.elements[5]));

//     radar_data_ptr->HeaderObjList.iNumOfUsedObjects = (sint8) (atoi(row.elements[6]));
//     for(int i = 0; i < BSW_NR_PRIVOBJECTS; i++)
//         radar_data_ptr->HeaderObjList.iDistXSortedObjectList[i] = (sint8) (atoi(row.elements[7+i]));

//     for(int i = 0; i < BSW_OBJECT_NUM_MAX; i++){
//         radar_data_ptr->Objects[i].ObjectId = (uint8) (atoi(row.elements[32+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fDistX = (float32) (atof(row.elements[33+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fDistXStd = (float32) (atof(row.elements[34+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fDistY = (float32) (atof(row.elements[35+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fDistYStd = (float32) (atof(row.elements[36+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fVrelX = (float32) (atof(row.elements[37+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fVrelXStd = (float32) (atof(row.elements[38+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fVrelY = (float32) (atof(row.elements[39+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fVrelYStd = (float32) (atof(row.elements[40+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fArelX = (float32) (atof(row.elements[41+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fArelXStd = (float32) (atof(row.elements[42+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fArelY = (float32) (atof(row.elements[43+37*i]));
//         radar_data_ptr->Objects[i].Kinematic.fArelYStd = (float32) (atof(row.elements[44+37*i]));
//         radar_data_ptr->Objects[i].Geometry.fWidth = (float32) (atof(row.elements[45+37*i]));
//         radar_data_ptr->Objects[i].Geometry.fLength = (float32) (atof(row.elements[46+37*i]));
//         radar_data_ptr->Objects[i].Geometry.fOrientation = (float32) (atof(row.elements[47+37*i]));
//         radar_data_ptr->Objects[i].Geometry.fOrientationStd = (float32) (atof(row.elements[48+37*i]));
//         radar_data_ptr->Objects[i].Geometry.fOrientationValid = (float32) (atof(row.elements[49+37*i]));
//         radar_data_ptr->Objects[i].Attributes.eDynamicProperty = (uint8) (atoi(row.elements[50+37*i]));
//         radar_data_ptr->Objects[i].Attributes.uiStoppedConfidence = (uint8) (atoi(row.elements[51+37*i]));
//         radar_data_ptr->Objects[i].Attributes.eAbsMovingState = (uint8) (atoi(row.elements[52+37*i]));
//         radar_data_ptr->Objects[i].Attributes.eClassification = (uint8) (atoi(row.elements[53+37*i]));
//         radar_data_ptr->Objects[i].Attributes.uiClassConfidence = (uint8) (atoi(row.elements[54+37*i]));
//         radar_data_ptr->Objects[i].General.fLifeTime = (float32) (atof(row.elements[55+37*i]));
//         radar_data_ptr->Objects[i].General.fTimeStamp = (uint32) (atoi(row.elements[56+37*i]));
//         radar_data_ptr->Objects[i].General.eObjMaintenanceState = (uint8) (atoi(row.elements[57+37*i]));
//         radar_data_ptr->Objects[i].General.cObjMergingID = (uint8) (atoi(row.elements[58+37*i]));
//         radar_data_ptr->Objects[i].Qualifiers.fProbabilityOfExistence = (float32) (atof(row.elements[59+37*i]));
//         radar_data_ptr->Objects[i].Qualifiers.ucObstacleProbability = (uint8) (atoi(row.elements[60+37*i]));
//         radar_data_ptr->Objects[i].Qualifiers.uMeasuredTargetFrequencyNear = (uint8) (atoi(row.elements[61+37*i]));
//         radar_data_ptr->Objects[i].Qualifiers.uMeasuredTargetFrequencyFar = (uint8) (atoi(row.elements[62+37*i]));
//         radar_data_ptr->Objects[i].SensorSpecific.fRCS = (float32) (atof(row.elements[63+37*i]));
//         radar_data_ptr->Objects[i].SensorSpecific.bCamConfirmed = (boolean) (atoi(row.elements[64+37*i]));
//         radar_data_ptr->Objects[i].Legacy.fMaxAccelY = (float32) (atof(row.elements[65+37*i]));
//         radar_data_ptr->Objects[i].Legacy.fRCSTargetThresholdUncomp = (float32) (atof(row.elements[66+37*i]));
//         radar_data_ptr->Objects[i].Legacy.uiLifeTime = (uint16) (atof(row.elements[67+37*i]));
//         radar_data_ptr->Objects[i].Legacy.eDynamicSubProperty = (uint16) (atof(row.elements[68+37*i]));
//     }
// }

/**
 * @brief       populates a LidarObjectList_t object from corresponding row in CSV file
 * 
 * After reading one row of CSV file, this function is used to fill each element of
 * Lidar_input_lists from corresponding row of CSV file
 * 
 * @param[in]   row one row of CSV file
 * @param[in]   lidar_data_ptr pointer to Lidar_input_lists variable that is being populated
 * 
 * @return      Nothing is returned
 * 
*/
/*
static void fillLidarObjectListFromCSV(CSVRow row, Lidar_input_lists* lidar_data_ptr){
    lidar_ts.timestamp[lidar_ts.idx++] = (uint32) (atoi(row.elements[0]));
   // lidar_data_ptr->Lidarobjlists->Objects[0].ID= (uint32) (atoi(row.elements[1]));
    lidar_data_ptr->Lidarobjlists->timestamp = (double) (atoi(row.elements[2]));
    // Set simulation timestamp
    //idar_data_ptr->sSigHeader.uiMeasurementCounter = (uint16) (atoi(row.elements[3]));
    //lidar_data_ptr->sSigHeader.uiCycleCounter = (uint16) (atoi(row.elements[4]));
    //lidar_data_ptr->sSigHeader.eSigStatus = (uint8) (atoi(row.elements[5]));

    lidar_data_ptr->Lidarobjlists->uNumOfUsedObjects = (uint16) (atoi(row.elements[6]));
    //for(int i = 0; i < BSW_NR_PRIVOBJECTS; i++)
       // lidar_data_ptr->HeaderObjList.iDistXSortedObjectList[i] = (sint8) (atoi(row.elements[7+i]));

    for(int i = 0; i < BSW_OBJECT_NUM_MAX; i++){
        lidar_data_ptr->Lidarobjlists->Objects[i].ID = (uint8) (atoi(row.elements[32+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fDist.data[0] = (float32) (atof(row.elements[33+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fCovDist.data[0][0] = (float32) (atof(row.elements[34+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fDist.data[1] = (float32) (atof(row.elements[35+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fCovDist.data[1][1] = (float32) (atof(row.elements[36+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fVel.data[0] = (float32) (atof(row.elements[37+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fCovVel.data[0][0] = (float32) (atof(row.elements[38+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fVel.data[1] = (float32) (atof(row.elements[39+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fCovVel.data[1][1] = (float32) (atof(row.elements[40+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fAcc.data[0] = (float32) (atof(row.elements[41+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fCovAcc.data[0][0] = (float32) (atof(row.elements[42+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fAcc.data[1] = (float32) (atof(row.elements[43+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Kinematic.fCovAcc.data[1][1] = (float32) (atof(row.elements[44+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Geometry.width = (float64) (atof(row.elements[45+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Geometry.length = (float64) (atof(row.elements[46+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Geometry.height =0; 
        lidar_data_ptr->Lidarobjlists->Objects[i].Geometry.theta = (float64) (atof(row.elements[47+37*i]));
       // lidar_data_ptr->Objects[i].Lidar_Geometry_t.fOrientationStd = (float32) (atof(row.elements[48+37*i]));
       // lidar_data_ptr->Objects[i].Lidar_Geometry_t.fOrientationValid = (float32) (atof(row.elements[49+37*i]));
       // lidar_data_ptr->Objects[i].Lidar_Geometry_t.eDynamicProperty = (uint8) (atoi(row.elements[50+37*i]));
       // lidar_data_ptr->Objects[i].Lidar_Attributes_t.uiStoppedConfidence = (uint8) (atoi(row.elements[51+37*i]));
       // lidar_data_ptr->Objects[i].Lidar_Attributes_t.eAbsMovingState = (uint8) (atoi(row.elements[52+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Attributes.Type = (uint8) (atoi(row.elements[53+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Attributes.SubType =0;
        lidar_data_ptr->Lidarobjlists->Objects[i].Attributes.confidence = (float64) (atoi(row.elements[54+37*i]));
        //lidar_data_ptr->Objects[i].General.fLifeTime = (float32) (atof(row.elements[55+37*i]));
        //lidar_data_ptr->Objects[i].General.fTimeStamp = (uint32) (atoi(row.elements[56+37*i]));
        //lidar_data_ptr->Objects[i].General.eObjMaintenanceState = (uint8) (atoi(row.elements[57+37*i]));
        //lidar_data_ptr->Objects[i].General.cObjMergingID = (uint8) (atoi(row.elements[58+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Qualifiers.fProbabilityOfExistence = (float32) (atof(row.elements[59+37*i]));
        lidar_data_ptr->Lidarobjlists->Objects[i].Qualifiers.ucObstacleProbability = (uint8) (atoi(row.elements[60+37*i]));
        //lidar_data_ptr->Objects[i].Lidar_Qualifiers_t.uMeasuredTargetFrequencyNear = (uint8) (atoi(row.elements[61+37*i]));
        //lidar_data_ptr->Objects[i].Lidar_Qualifiers_t.uMeasuredTargetFrequencyFar = (uint8) (atoi(row.elements[62+37*i]));
       // lidar_data_ptr->Objects[i].SensorSpecific.fRCS = (float32) (atof(row.elements[63+37*i]));
        //lidar_data_ptr->Objects[i].SensorSpecific.bCamConfirmed = (boolean) (atoi(row.elements[64+37*i]));
        //lidar_data_ptr->Objects[i].Legacy.fMaxAccelY = (float32) (atof(row.elements[65+37*i]));
        //lidar_data_ptr->Objects[i].Legacy.fRCSTargetThresholdUncomp = (float32) (atof(row.elements[66+37*i]));
        //lidar_data_ptr->Objects[i].Legacy.uiLifeTime = (uint16) (atof(row.elements[67+37*i]));
       // lidar_data_ptr->Objects[i].Legacy.eDynamicSubProperty = (uint16) (atof(row.elements[68+37*i]));
    }
}
*/
/**
 * @brief       populates a VehDyn_t object from corresponding row in CSV file
 * 
 * After reading one row of CSV file, this function is used to fill each element of
 * VehDyn_t from corresponding row of CSV file
 * 
 * @param[in]   row one row of CSV file
 * @param[in]   veh_data_row_ptr pointer to VehDyn_t that is being populated
 * 
 * @return      Nothing is returned
 * 
*/
static void fillVehDynFromCSV(CSVRow row, VehDyn_t* veh_data_row_ptr){
    // Set simulation timestamp from record timestamp
    ego_veh_ts.timestamp[ego_veh_ts.idx++] = (uint32) (atoi(row.elements[0]));
    veh_data_row_ptr->uiVersionNumber = (uint32) (atoi(row.elements[1]));

    veh_data_row_ptr->sSigHeader.uiTimeStamp = (uint32) (atoi(row.elements[2]));
    veh_data_row_ptr->sSigHeader.uiMeasurementCounter = (uint16) (atoi(row.elements[3]));
    veh_data_row_ptr->sSigHeader.uiCycleCounter = (uint16) (atoi(row.elements[4]));
    veh_data_row_ptr->sSigHeader.eSigStatus = (uint8) (atoi(row.elements[5]));

    veh_data_row_ptr->Longitudinal.MotVar.Velocity = (float) (atof(row.elements[6]));
    veh_data_row_ptr->Longitudinal.MotVar.Accel = (float) (atof(row.elements[7]));
    veh_data_row_ptr->Longitudinal.MotVar.varVelocity = (float) (atof(row.elements[8]));
    veh_data_row_ptr->Longitudinal.MotVar.varAccel = (float) (atof(row.elements[9]));
    veh_data_row_ptr->Longitudinal.VeloCorr.corrFact = (float) (atof(row.elements[10]));
    veh_data_row_ptr->Longitudinal.VeloCorr.corrVar = (float) (atof(row.elements[11]));
    veh_data_row_ptr->Longitudinal.VeloCorr.corrVelo = (float) (atof(row.elements[12]));
    veh_data_row_ptr->Longitudinal.VeloCorr.corrVeloVar = (float) (atof(row.elements[13]));
    veh_data_row_ptr->Longitudinal.VeloCorr.minVelo = (float) (atof(row.elements[14]));
    veh_data_row_ptr->Longitudinal.VeloCorr.maxVelo = (float) (atof(row.elements[15]));
    //FIXME: I assume we want byte 0 and 1 not char 0 and 1. Make sure this is correct 
    // for boolean types
    veh_data_row_ptr->Longitudinal.VeloCorr.corrQual = (boolean) (atoi(row.elements[16]));
    veh_data_row_ptr->Longitudinal.VeloCorr.bRollerTestBench = (boolean) (atoi(row.elements[17]));
    veh_data_row_ptr->Longitudinal.AccelCorr.corrAccel = (float) (atof(row.elements[18]));
    veh_data_row_ptr->Longitudinal.AccelCorr.corrAccelVar = (float) (atof(row.elements[19]));

    veh_data_row_ptr->Lateral.YawRate.YawRate = (float) (atof(row.elements[20]));
    veh_data_row_ptr->Lateral.YawRate.Variance = (float) (atof(row.elements[21]));
    veh_data_row_ptr->Lateral.YawRate.Quality = (float) (atof(row.elements[22]));
    //FIXME: It seems that this element is not a float type value in CSV file
    veh_data_row_ptr->Lateral.OffCompStWheelAngle = (float) (atof(row.elements[23]));
    veh_data_row_ptr->Lateral.Curve.Curve = (float) (atof(row.elements[24]));
    //FIXME: It seems that this element is not a float type value in CSV file
    veh_data_row_ptr->Lateral.Curve.C1 = (float) (atof(row.elements[25]));
    veh_data_row_ptr->Lateral.Curve.Gradient = (float) (atof(row.elements[26]));
    veh_data_row_ptr->Lateral.Curve.varC0 = (float) (atof(row.elements[27]));
    //FIXME: It seems that this element is not a float type value in CSV file
    veh_data_row_ptr->Lateral.Curve.varC1 = (float) (atof(row.elements[28]));
    veh_data_row_ptr->Lateral.Curve.Quality = (float) (atof(row.elements[29]));
    veh_data_row_ptr->Lateral.Curve.CrvError = (float) (atof(row.elements[30]));
    veh_data_row_ptr->Lateral.Curve.CrvConf = (boolean) (atoi(row.elements[31]));
    veh_data_row_ptr->Lateral.DrvIntCurve.Curve = (float) (atof(row.elements[32]));
    veh_data_row_ptr->Lateral.DrvIntCurve.Variance = (float) (atof(row.elements[33]));
    veh_data_row_ptr->Lateral.DrvIntCurve.Gradient = (float) (atof(row.elements[34]));
    veh_data_row_ptr->Lateral.Accel.LatAccel = (float) (atof(row.elements[35]));
    veh_data_row_ptr->Lateral.Accel.Variance = (float) (atof(row.elements[36]));
    veh_data_row_ptr->Lateral.SlipAngle.SideSlipAngle = (float) (atof(row.elements[37]));
    veh_data_row_ptr->Lateral.SlipAngle.Variance = (float) (atof(row.elements[38]));


    veh_data_row_ptr->MotionState.MotState = (boolean) (atoi(row.elements[39]));
    veh_data_row_ptr->MotionState.Confidence = (float) (atof(row.elements[40]));

    veh_data_row_ptr->Legacy.YawRateMaxJitter = (float) (atof(row.elements[41]));
    veh_data_row_ptr->Legacy.bStandStill = (boolean) (atoi(row.elements[42]));

    for(int i = 0; i < 12; i++){
        veh_data_row_ptr->State[i] = (VEDIoStateTypes_t) (atoi(row.elements[43+i]));
    }        
}

/**
 * @brief       reads next line of CSV file
 * 
 * Next line of file is read and a CSVRow variable is filled with it. It must be 
 * preallocated
 * 
 * @param[in]   fptr_ptr to the file pointer
 * @param[in]   row_ptr to a CSVRow that is populated
 * 
 * @return      Nothing is returned
*/
static void readNextLine(FILE** fptr_ptr, CSVRow* row_ptr){
    char line[LINE_BUFF_SIZE];
    int idx = 0;
    int elem_len = 0;
    int line_idx = 0;
    char ch;
    // Read whole current line
    while(TRUE){
        ch = fgetc(*fptr_ptr);
        if(feof(*fptr_ptr))
            return;
        if(ch == '\n'){
            if(idx < LINE_BUFF_SIZE)
                line[idx] = '\0';
            break;
        } else {
            if(idx < LINE_BUFF_SIZE)
                line[idx++] = ch;
            else
                break;
        }
    }
    // extract elements
    for(int elem_idx = 0; elem_idx < row_ptr->len; elem_idx++){
        elem_len = getNextElement(&line[line_idx], &row_ptr->elements[elem_idx]);
        if(elem_len > 0){
            line_idx += elem_len+1;
        } else {
            LogWrite("Unable to read %d element\n", elem_idx);
            break;
        }     
    }
}

/**
 * @brief       moves file pointer to next line
 * 
 * File pointer will be change from previous position to the next one. 
 * Next line is characterized by '\n' character
 * 
 * @param[in]   fptr_ptr to the file pointer
 * 
 * @return      Nothing is returned
*/
static void moveFilePtrToNextLine(FILE** fptr_ptr){
    char ch;
    while(!feof(*fptr_ptr)){
        ch = fgetc(*fptr_ptr);
        if(ch == '\n')
            return;
    }
}

/**
 * @brief       read next element in csv
 * 
 * reads next element in a csv format string. It is assumed that elements
 * are separeted with ',' character.
 *  
 * @param[in]   line csv string that is parsed
 * @param[in]   element_ptr pointer to string that holds extracted element. Must be allocated beforehand
 * 
 * @return      length of element is returned
*/
static int getNextElement(char* line, char** element_ptr){
    // TODO: Set 31 from CSV_ELEMENT_SIZE
    char buff[CSV_ELEMENT_SIZE];
    static char format[] = "%31[^,\n]";
    int len = sscanf(line, format, buff);
    if (len > 0)
        strcpy(*element_ptr, buff);
    // printf("%s|%s\n", buff, *element_ptr);
    return len * strlen(buff);
}

/**
 * @brief       initialize SimTimestamp struct
 * 
 * A SimTimestamp object is initialized. Index is set to 0 and len is set to 
 * input argument size. timestamp array is intialized on the heap, so must be 
 * freed manually.
 *  
 * @param[in]   sim_timestamp constant pointer to the SimTimestamp struct
 * @param[in]   size timestamp array size
 * @param[out]  statue error type
 * 
 * @return      error type is returned
*/
uint32 initSimTimestamp(SimTimestamp* const sim_timestamp, const uint32 size){
    uint32 status = TUEOBJFUSN_ERROR_NOERROR;
    sim_timestamp->timestamp = malloc(size * sizeof(uint32));
    #if (TUE_PRV_RUNTIME_ERROR_CHECK == STD_ON)
        if(sim_timestamp == NULL){
            status = TUEOBJFUSN_ERROR_NULL_POINTER_EXCEPTION;
        }
    #endif
    sim_timestamp->len = size;
    sim_timestamp->idx = 0u;
    return status;
}

void readSimTimestamp(char* file_name, SimTimestamp* const ts){
    uint32 temp;
    FILE* fptr = fopen(file_name, "r");
    if(fptr == NULL){
        LogWrite("Unable to open %s file!\n", file_name);
        return;
    }

    while(TRUE){
        fscanf(fptr, "%u", &temp);
        if(feof(fptr))
            break;
        ts->timestamp[ts->idx++] = temp;
    }

    if(ts->len != ts->idx)
        LogWrite("Simulation timstamp size is not matching!\n");
    ts->idx = 0;

    fclose(fptr);
}

void checkFinishedStatus(const SimTimestamp* const ts_arr, const uint32 size, boolean* const status){
    for(int i=0; i<size; i++)
        status[i] = isDataFinished(ts_arr+i);
}

boolean isDataFinished(const SimTimestamp* const ts){
    return (ts->idx >= ts->len) ? TRUE : FALSE;
}

uint32 getMinTsWithMask(const SimTimestamp* const ts_arr, const uint32 size, const boolean* const fin_status){
    uint32 min_val = ts_arr[0].timestamp[ts_arr[0].idx];
    uint32 min_idx = 0;
    for(int i=1; i<size; i++){
        if(fin_status[i]) // This timestamp has been consumed
            continue;
        if(ts_arr[i].timestamp[ts_arr[i].idx] <= min_val){
            min_val = ts_arr[i].timestamp[ts_arr[i].idx];
            min_idx = i;
        }
    }
    return min_idx;
}
